/**
 * Game of Life
 * 
 */
package gridlife.color;


/**
 * Imports
 * 
 */
import gridlife.core.GridLifeConstants;
import java.util.HashMap     ;
import java.util.Comparator  ;
import java.util.Set         ;
import java.util.Iterator    ;
import java.util.Arrays      ;
import java.io.File          ;
import java.io.FileReader    ;
import java.io.BufferedReader;
import java.awt.Color        ;



/**
 * This class represents a colorset, which is number of colors stored as a unit.
 * 
 * @author Ulf Wagemann
 */
public class GridLifeColorSet 
{
 private String m_sColorSetName ;
 private int    m_iColorAmount  ;
 private int    m_iMaxEncodedAge;
 
 private HashMap<Integer, GridLifeColorRecord> m_tColorTable  ;
 private GridLifeColorRecord[]                 m_tColorRecords;

 private Color m_tDefaultCellFillColor;
 
 
 
 /**
  * Constructor. Loads a color set file
  * 
  * @param pa_sFileName 
  */ 
 public GridLifeColorSet(File pa_tFile)
 {
  m_sColorSetName  = "";
  m_iColorAmount   =  0;
  m_iMaxEncodedAge =  0;
  m_tColorRecords  = null;
  
  m_tDefaultCellFillColor = new Color(GridLifeConstants.GOL_CELL_FILL_COLOR); 
  if (null != (m_tColorTable = new HashMap<Integer, GridLifeColorRecord>()))
  {
   load(pa_tFile);
  }
 }
 
 
  
 /**
  * We do not want to eat up memory, since we store own datastructures we need to remove them from the
  * hashmap and mark then as gone for the GC.
  * 
  * dispose() should be called when this shapeset is not needed anymore.
  * 
  */
 public final void dispose()
 {
  if (null != m_tColorTable) {m_tColorTable.clear();}
 }
 
 
  
 /**
  * Loads a shape set file. Format:
  * 
  * Line 1    : @GOL_COLORSET@<name>
  * Line 2-...: Colorstring  (@see GridLifeColorRecord.java)
  * 
  * Lines starting with "#" are treated as comments.
  * 
  * @param pa_sFileName   Filename 
  */
 private void load(File pa_tFile)
 {
  FileReader           lc_tInputReader         = null;
  BufferedReader       lc_tBufferedInputReader = null;
  String               lc_sInputLine           = null;
  String               lc_sCheckLine           = null;
  GridLifeColorRecord  lc_tRecord              = null;
  
  Iterator<Integer>               lc_tIterator = null;
  Set<Integer>                    lc_tNames    = null;

   
  int lc_iPosition = 0;
  int lc_iAge      = 0;
  int lc_iCount    = 0;
  
  m_iColorAmount = 0;
  if (null != pa_tFile && null != m_tColorTable) 
  {
   if (true == pa_tFile.isFile())   
   {
    try
    {
     if (null != (lc_tInputReader = new FileReader(pa_tFile)))   
     {
      if (null != (lc_tBufferedInputReader = new BufferedReader(lc_tInputReader)))   
      {
       while (null != (lc_sInputLine = lc_tBufferedInputReader.readLine()))    
       {
        //
        // remove white spaces, check for commentary lines
        //
        if (null != (lc_sCheckLine = lc_sInputLine.trim()))
        {
         if (false == lc_sCheckLine.isEmpty())
         {
          if (false == lc_sCheckLine.startsWith(GridLifeConstants.GOL_FILE_COLOR_COMMENT))
          {
           //
           // check for color header
           //
           if (true == lc_sCheckLine.startsWith(GridLifeConstants.GOL_FILE_COLOR_HEADER)) 
           {
            lc_iPosition =  lc_sCheckLine.indexOf(GridLifeConstants.GOL_FILE_COLOR_HEADER) + GridLifeConstants.GOL_FILE_COLOR_HEADER.length();
            m_sColorSetName = lc_sCheckLine.substring(lc_iPosition);
           }
           //
           // should be a color line
           //
           else
           {
            if (null != (lc_tRecord = new GridLifeColorRecord(lc_sCheckLine))) 
            {
             lc_iAge = lc_tRecord.getAge(); 
             if (lc_iAge > m_iMaxEncodedAge)
             {
              m_iMaxEncodedAge = lc_iAge; 
             }
             m_tColorTable.put(new Integer(lc_iAge), lc_tRecord);
             m_iColorAmount++;                        
            }
           }
          }
         }
        }
       } // while
       
       //
       // create and sort our array for performance improvement in getColor()
       //
       if (null != (lc_tNames = m_tColorTable.keySet()))
       {
        if (null != (lc_tIterator = lc_tNames.iterator()))
        {
         if (null != (m_tColorRecords = new GridLifeColorRecord[m_tColorTable.values().size()]))
         {
          while (lc_tIterator.hasNext()) 
          {
           if (null != (lc_tRecord = m_tColorTable.get(lc_tIterator.next())))
           {
            m_tColorRecords[lc_iCount++] = lc_tRecord; 
           }
          } // while
          
          Arrays.sort(m_tColorRecords, new Comparator<GridLifeColorRecord>() 
                                {
                                 public int compare(GridLifeColorRecord o1, GridLifeColorRecord o2)
                                 { 
                                  if      (o1.getAge() < o2.getAge() ) return -1;
                                  else if (o1.getAge() == o2.getAge()) return 0 ;
                                  else return 1;
                                 }
                                });         
         }
        }
       }
       
       // done!
      }
     }
    }
    catch (Exception lc_tException)
    {}

    //
    // Closing file stuff at this position to make sure that in any case we close them!
    //
    try {if (null != lc_tBufferedInputReader) lc_tBufferedInputReader.close(); } catch (Exception lc_tException) {}   
    try {if (null != lc_tInputReader        ) lc_tInputReader.close();         } catch (Exception lc_tException) {}   
   }   
  }
 }

 
 
 /**
  * Returns the closest color for the given age
  * 
  * @param pa_iAge    Age
  * @return  GridLifeConstants.GOL_CELL_FILL_COLOR in case of error or default, corresponding color otherwise
  */
 public final Color getColor(int pa_iAge)
 {
  int     lc_iCurrentPos = 0;
  int     lc_iFoundPos   = 0;
  
  boolean lc_bFound      = false;
 
  GridLifeColorRecord lc_tRecord = null;
  
  if (null != m_tColorRecords)
  {
   if (m_tColorRecords.length > 0)
   {
    while (false == lc_bFound && lc_iCurrentPos < m_tColorRecords.length)
    {
     if (null != (lc_tRecord = m_tColorRecords[lc_iCurrentPos]))
     {
      if (lc_tRecord.getAge() == pa_iAge)
      {
       lc_bFound    = true          ;
       lc_iFoundPos = lc_iCurrentPos;
      }
      else if (lc_tRecord.getAge() < pa_iAge)
      {
       lc_iCurrentPos++; 
      }
      else // we found the first greater value, but check for beginning of array
      {
       lc_bFound    = true;
       lc_iFoundPos = (0 < lc_iCurrentPos ? lc_iCurrentPos - 1 : lc_iCurrentPos);      
      }
     }
     else
     {
      lc_iCurrentPos++; 
     }
    } // while
    
    //
    // return the found element's color if existing, if not return the last element's color
    //
    return (true == lc_bFound ? m_tColorRecords[lc_iFoundPos].getBgColor() 
                              : m_tColorRecords[m_tColorRecords.length - 1].getBgColor()); 
   }
  }
  return m_tDefaultCellFillColor; 
 }
 
 
 
 /**
  * Returns the color set name
  * 
  * @return    null in case of error, set name otherwise
  */
 public final String getSetName() {return m_sColorSetName;}
 
 

 /**
  * Returns the sorted array of color records
  * 
  * @return  null in case of error, array otherwisw
  */
 public final GridLifeColorRecord[] getSet() {return m_tColorRecords;}
 

 
 /**
  * Gets the max encoded age
  * 
  * @return max age a color is defined for
  */
 public final int getMaxEncodedAge()
 {
  return m_iMaxEncodedAge; 
 }
 
 
 
 /**
  * toString() method
  * 
  * @return   colorset name
  */
 @Override
 public String toString()
 {
  return m_sColorSetName;
 }
 
 
 
 /**
  * Returns a sorted array of ages for this color set
  * 
  * @return 
  */
 public final synchronized int[] getAges()
 {
  Set<Integer>        lc_tKeys     = null;
  Iterator<Integer>   lc_tIterator = null;  
  Integer             lc_tAge      = null;
  int[]               lc_tResult   = null;
 
  int lc_iSize  = 0;
  int lc_iCount = 0;
  
  if (null != m_tColorTable)
  {
   if (0 < (lc_iSize = m_tColorTable.size()))
   {
    if (null != (lc_tKeys = m_tColorTable.keySet()))
    {
     if (null != (lc_tIterator = lc_tKeys.iterator()))
     {
      synchronized(lc_tIterator)
      {
       if (null != (lc_tResult = new int[lc_iSize]))
       {
        while (lc_tIterator.hasNext())
        {
         if (null != (lc_tAge = lc_tIterator.next())) 
         {
          lc_tResult[lc_iCount++] =  lc_tAge.intValue();
         }
        } // while
        java.util.Arrays.sort(lc_tResult);
       }
      } 
     }
    }
   }
  }
  return lc_tResult;
   
 }
 
 
 /**
  * Returns the amount of stored colors
  * 
  * @return  amount of storec colors
  */
 public final int getAmount() {return (null != m_tColorRecords ? m_tColorRecords.length : 0);}
}
