/**
 * Game of Life
 * 
 */
package gridlife.core;



/**
 * Imports
 * 
 */
import java.util.concurrent.ConcurrentHashMap;
import java.util.Set     ;
import java.util.Iterator;

import gridlife.color.GridLifeColorManager;



/**
 * Keeps track of a certain amount of history records. 
 * These records are kept in a ConcurrentHashMap to avoid access problems caused by multithreading.
 * 
 * @author Ulf Wagemann
 */
public class GridLifeAgeHistogramManager 
{
 private ConcurrentHashMap<Integer,Integer> m_tAgeRecords  ;
 private GridLifeColorManager               m_tColorManager;
 
 
 /**
  * Constructor
  * 
  */
 public GridLifeAgeHistogramManager(GridLifeColorManager pa_tColorManager)
 {
  m_tColorManager = pa_tColorManager;
  m_tAgeRecords   = new ConcurrentHashMap<Integer,Integer>();
 }
 
 
 
 /**
  * Returns the amount of stored records
  * 
  * @return  -1 in case of error, the amount of stored records otherwise
  */
 public final int getRecordAmount() {return (null != m_tAgeRecords ? m_tAgeRecords.size() : -1);}
 
 
 
 /**
  * Increases the value for an age. If the age is not listed yet, is is inserted with an amount of 1
  * 
  * @param pa_iAge   age
  */
 public final void add(int pa_iAge)
 {
  Integer lc_tValue    = null;
  int     lc_iNewValue = 1   ;
  
  if (null != m_tAgeRecords)  
  {
   if (null != (lc_tValue = m_tAgeRecords.get(pa_iAge)))
   {
    lc_iNewValue = 1 + lc_tValue.intValue();
   }
   m_tAgeRecords.put(new Integer(pa_iAge), new Integer(lc_iNewValue)); 
  }
 }
 
  
 
 /**
  * Increases the value for an age. If the age is not listed yet, is is inserted with an amount of 1
  * 
  * @param pa_iAge   age
  */
 public final void decrease(int pa_iAge)
 {
  Integer lc_tValue    = null ;
  int     lc_iNewValue = 1    ;
  
  if (null != m_tAgeRecords)  
  {
   if (null != (lc_tValue = m_tAgeRecords.get(pa_iAge)))
   {
    lc_iNewValue = lc_tValue.intValue();
   }
   lc_iNewValue--;
   if (0 > lc_iNewValue) {lc_iNewValue = 0;}
   
   m_tAgeRecords.put(new Integer(pa_iAge), new Integer(lc_iNewValue)); 
  }
 }
 
 
 /**
  * Adds the given amount for an age. If the age is not listed yet, is is inserted with an amount of pa_iAmount
  * 
  * @param pa_iAge   age
  * @param pa_iAmount
  */
 public final void add(int pa_iAge, int pa_iAmount)
 {
  Integer lc_tValue    = null;
  int     lc_iNewValue = 1   ;
  
  if (null != m_tAgeRecords)  
  {
   if (null != (lc_tValue = m_tAgeRecords.get(pa_iAge)))
   {
    lc_iNewValue = lc_tValue.intValue();
   }
   lc_iNewValue += pa_iAmount;
   m_tAgeRecords.put(new Integer(pa_iAge), new Integer(lc_iNewValue)); 
  }
 }
 
 
 
 /**
  * Rerieves the amount for an age
  * 
  * @param pa_iAge
  * @return -1 in case of error, otherwise amount for the given age
  */
 public final int get(int pa_iAge) 
 {
  Integer lc_iValue = null;
  
  if (null != m_tAgeRecords)  
  {
   if (null != (lc_iValue = m_tAgeRecords.get(pa_iAge)))
   {
    return lc_iValue.intValue();
   }
  }
  return -1; 
 }
  
  

 
 /**
  * Resets the mapping
  * 
  */
 public final void reset()
 {
  if (null != m_tAgeRecords)
  {
   m_tAgeRecords.clear(); 
  }
 }
 
 
 
 /**
  * Returns the age map
  * 
  * @return   age map
  */
 public final ConcurrentHashMap<Integer, Integer> getMap() {return m_tAgeRecords;}
 
 
 
 
 
 /**
  * Returns a sorted array of the keys
  * 
  */
 public final synchronized int[] getKeys()
 {
  Set<Integer>      lc_tKeys     = null;
  Iterator<Integer> lc_tIterator = null;  
  Integer           lc_tAge      = null;
  int[]             lc_tResult   = null;
  
  int lc_iSize  = 0;
  int lc_iCount = 0;
  
  if (null != m_tAgeRecords)
  {
   if (null != (lc_tKeys = m_tAgeRecords.keySet()))
   {
    if (0 < (lc_iSize = lc_tKeys.size()))
    {
     if (null != (lc_tIterator = lc_tKeys.iterator()))
     {
      if (null != (lc_tResult = new int[lc_iSize]))
      {
       while (null !=lc_tIterator && lc_tIterator.hasNext())
       {
        if (null != (lc_tAge = lc_tIterator.next())) 
        {
         //
         // due to asynchronous execution, make sure we don't get problems when the call happens during changes
         // of the computing threads
         //
         if (lc_iCount < lc_iSize)
         {
          lc_tResult[lc_iCount++] =  lc_tAge.intValue();
         }
        }
       } // while
       java.util.Arrays.sort(lc_tResult);
      } 
     }
    }
   }
  }
  return lc_tResult;
 }
 
 
 /**
  * Retrieves the amount of cells from the records
  * 
  * @return   amount of cells
  */
 public final int sum()
 {
  Set<Integer>      lc_tKeys     = null;
  Iterator<Integer> lc_tIterator = null;  
  Integer           lc_tAge      = null;
  Integer           lc_tRecords  = null;
  int               lc_iResult   = 0   ;
 
  if (null != m_tAgeRecords)
  {
   if (0 < m_tAgeRecords.size())
   {
    if (null != (lc_tKeys = m_tAgeRecords.keySet()))
    {
     if (null != (lc_tIterator = lc_tKeys.iterator()))
     {
      {        
       while (null != lc_tIterator && lc_tIterator.hasNext())
       {
        if (null != (lc_tAge = lc_tIterator.next())) 
        {
         if (null != (lc_tRecords = m_tAgeRecords.get(lc_tAge))) 
         {
          lc_iResult +=  lc_tRecords.intValue();
         }
        }
       } // while
      } 
     }
    }
   }
  }
  return lc_iResult;   
 }

 
 
 /**
  * Prepares for import. Creates age entries according to the  colormanager settings.
  * 
  */
 public final void prepareImport()
 {
  int[] lc_tAges   = null;
  int   lc_iLength = 0   ;
  int   lc_iI      = 0   ;
  
  if (null != m_tColorManager)
  {
   if (null != (lc_tAges = m_tColorManager.getActiveAges()))
   {
    if (0 < (lc_iLength = lc_tAges.length))
    {
     reset(); 
     
     for (lc_iI = 0; lc_iI < lc_iLength; lc_iI++)
     {
      m_tAgeRecords.put(lc_tAges[lc_iI], 0); 
     }
    }
   }
  }
 }
 
 
 
/**
  * Imports age data from the given manager. Thereby, it sorts ages according to the colormanager settings
  * 
  */
 public final void importData(GridLifeAgeHistogramManager pa_tHistogramManager)
 {
  ConcurrentHashMap<Integer, Integer> lc_tMap = null;
  
  Set<Integer>      lc_tKeys     = null;
  Iterator<Integer> lc_tIterator = null;
  
  Integer lc_tKey      = null;
  Integer lc_tOwnKey   = null;
  Integer lc_tValue    = null;
  Integer lc_tOwnValue = null;
          
  int lc_iOwnValue = 0;
  int lc_iNewValue = 0;
  
  if (null != pa_tHistogramManager)
  {
   if (null != (lc_tMap = pa_tHistogramManager.getMap()))
   {
    if (null != (lc_tKeys = lc_tMap.keySet()))
    {
     if (null != (lc_tIterator = lc_tKeys.iterator()))
     {
      {      
       //
       // iterate over map
       //       
       while (null != lc_tIterator && lc_tIterator.hasNext())
       {
        //
        // get age from import map
        //
        if (null != (lc_tKey = lc_tIterator.next())) 
        {
         //
         // get age value from import manager
         //
         if (null != (lc_tValue = pa_tHistogramManager.get(lc_tKey))) 
         {
          //
          // get own record for that age according to colormanager settings
          // 
           
          if (null != (lc_tOwnKey = m_tColorManager.getAgeClass(m_tColorManager.getActiveSetName(), lc_tKey, null)))
          {
           lc_iOwnValue = 0;
           if (null != (lc_tOwnValue = m_tAgeRecords.get(lc_tOwnKey)))
           {
            lc_iOwnValue = lc_tOwnValue.intValue();
           }
           lc_iNewValue = lc_iOwnValue + lc_tValue.intValue();
           m_tAgeRecords.put(lc_tOwnKey, new Integer(lc_iNewValue));
          }
         }
        }
       }   // while
      } 
      
     }
    }
   }
  }
 }

 
 
 /**
  * Transfers the current age records to the structure according to the given colorsets settings
  * 
  * @param pa_sSetName   color set name
  */
 public final void switchToNewColorset(String pa_sSetName, GridLifeCellMatrix pa_tCellMatrix)
 {
  int[] lc_tSetAges   = null;
  
  int lc_iI     = 0;
  int lc_iJ     = 0;
  int lc_iNewCellAmount = 0;
  int lc_iCellAge = 0;
  int lc_iCellAmount = 0;
  
  Integer lc_tCellAge        = null;
  Integer lc_tCellAgeAmount  = null;
  Integer lc_tNewAge         = null;
  
  if (null != m_tAgeRecords && null != m_tColorManager && null != pa_sSetName)
  {
   if (null != (lc_tSetAges = m_tColorManager.getAgesForSet(pa_sSetName)))
   {
    reset();
     
    for (lc_iI = 0; lc_iI < pa_tCellMatrix.getRows(); lc_iI++)         
    {
     for (lc_iJ = 0; lc_iJ < pa_tCellMatrix.getColumns(); lc_iJ++)         
     {
      lc_iCellAge = pa_tCellMatrix.getCell(lc_iJ, lc_iI);
      
      //
      // only care about living cells
      //
      if (0 != lc_iCellAge)
      {
       lc_tCellAge = new Integer(lc_iCellAge);
      
       //
       // get new age class for cell age
       //
       if (null != (lc_tNewAge = m_tColorManager.getAgeClass(pa_sSetName, lc_tCellAge, lc_tSetAges)))
       {
        lc_iCellAmount = 0;
        //
        // get eventually already existent amount of cells for current age
        //
        if (null != (lc_tCellAgeAmount = m_tAgeRecords.get(lc_tNewAge))) 
        {
         lc_iCellAmount =  lc_tCellAgeAmount.intValue();
        }
        
        //
        // compute new value
        //
        lc_iNewCellAmount = lc_iCellAmount + 1;
        m_tAgeRecords.put(lc_tNewAge, new Integer(lc_iNewCellAmount));
       }
      }
     }  
    }
   }
  }
 }
 

 
  
} // eoc
