/**
 * Game of Life
 * 
 */
package gridlife.core;



/**
 * Imports
 * 
 */
import gridlife.color.GridLifeColorManager;



/**
 * This class provides a thead for computing a given part of the cell matrix according to a given ruleset. 
 * GridLife uses up to 256 Threads to perform nicely.
 * 
 * @author Ulf Wagemann
 */
public class GridLifeComputingThread extends Thread
{
 private GridLifeProperties          m_tRuleset;
 private GridLifeAgeHistogramManager m_tAgeHistogramManager;
 
 private int[][] m_tWorkArray; 
 private int[][] m_tCellArray;
 
 private int m_iColumns;
 private int m_iRows   ;
 private int m_iStartX ;
 private int m_iStartY ;
 private int m_iAmount ;
 
 
 
 /**
  * Constructor
  * 
  * @param pa_tCells    cell array
  * @param pa_iCellx1   start column of area
  * @param pa_iCelly1   start row of area
  * @param pa_iCellx2   end column of area
  * @param pa_iCelly2   end row of area
  * @param pa_tRuleset  ruleset
  */
 public GridLifeComputingThread(int[][] pa_tCells, int pa_iCellx1, int pa_iCelly1, int pa_iCellx2, int pa_iCelly2, GridLifeProperties pa_tRuleset, GridLifeColorManager pa_tColorManager)
 {
  super();
  
  m_tRuleset   = pa_tRuleset;
  m_tCellArray = pa_tCells  ;
  m_iAmount    = 0          ;
  
  m_iStartX  = pa_iCellx1;
  m_iStartY  = pa_iCelly1;
  m_iColumns = (pa_iCellx2 - pa_iCellx1) + 1;  
  m_iRows    = (pa_iCelly2 - pa_iCelly1) + 1;  
  
  if (0 < m_iRows    && GridLifeConstants.GOL_MAX_ROWS    >= m_iRows &&  // we have an absolute amount of rows here, so use >=
      0 < m_iColumns && GridLifeConstants.GOL_MAX_COLUMNS >= m_iColumns) // we have an absolute amount of rows here, so use >=
  {
   m_tWorkArray = new int[m_iRows][m_iColumns];
  }
  
  m_tAgeHistogramManager = new GridLifeAgeHistogramManager(pa_tColorManager);
 }
 
 
 
 /**
  * Checks whether the given amount of neighbours applies to the given ruleset
  * 
  * @param pa_tRuleset     int array
  * @param pa_iNeighbours  amount of neighbours
  * @return  true, if the amount of neighbours applies to the ruleset positively, false otherwise
  */
 private boolean appliesToRule(int[] pa_iRules, int pa_iNeighbours)
 {
  int     lc_iLength  = 0   ;
  boolean lc_bApplies = false;
  
  if (null != pa_iRules)
  {
   if (0 < (lc_iLength = pa_iRules.length)) 
   {
    if (0 <= pa_iNeighbours && lc_iLength > pa_iNeighbours)
    {
     lc_bApplies = (pa_iRules[pa_iNeighbours] == 1);
    }
   }
  }
  return lc_bApplies;
 }
 
 
 
 /**
  * In this method, finally the GridLife algorithm is executed on cells according to the ruleset.
  * The new cell area is stored in the internal work array and afterwards copied back into the
  * original array.
  * 
  */
 @Override
 public final void run()
 {
  int lc_iRow    = 0;
  int lc_iColumn = 0;
  
  int lc_iOldValue   = 0;
  int lc_iNewValue   = 0;
  int lc_iNeighbours = 0;
  
  int[] lc_iRulesStayAlive   = null;
  int[] lc_iRulesBecomeAlive = null;
  
  m_iAmount = 0;
  
  if (null != m_tCellArray && null != m_tWorkArray & null != m_tRuleset) 
  {
   lc_iRulesStayAlive   = m_tRuleset.getStayAliveRules()  ;
   lc_iRulesBecomeAlive = m_tRuleset.getBecomeAliveRules();
   
   if (null != lc_iRulesStayAlive && null !=  lc_iRulesBecomeAlive)
   {
    for (lc_iRow = 0; lc_iRow < m_iRows; lc_iRow++) 
    {
     for (lc_iColumn = 0; lc_iColumn < m_iColumns; lc_iColumn++) 
     {
      lc_iOldValue   = m_tCellArray[m_iStartY + lc_iRow][m_iStartX + lc_iColumn];
      lc_iNeighbours = countNeighbours(m_iStartX + lc_iColumn, m_iStartY + lc_iRow);
      lc_iNewValue   = 0;
          
      //
      // handle dead cell according to ruleset
      //
      if (0 == lc_iOldValue)
      {
       if (true == appliesToRule(lc_iRulesBecomeAlive, lc_iNeighbours))
       {
        lc_iNewValue = 1; // birthday!
        m_iAmount++;
        
        if (null != m_tAgeHistogramManager) m_tAgeHistogramManager.add(1);
       }
      }
      //
      // handle living cell according to ruleset
      //
      else
      {
       if (false == appliesToRule(lc_iRulesStayAlive, lc_iNeighbours))
       {
        lc_iNewValue = 0; // neighbours can kill...
       }
       else
       {
        lc_iNewValue = lc_iOldValue + 1; // cells grow older :)  
        m_iAmount++;
        
        if (null != m_tAgeHistogramManager) m_tAgeHistogramManager.add(lc_iNewValue);        
       }  
      }
      m_tWorkArray[lc_iRow][lc_iColumn]= lc_iNewValue;
     }  
    }
   }
  }
 }
 
 
 /**
  * Counts the amount of neighbours for a given cell accoding to the ruleset
  * 
  * @param pa_iColumn   column of cell
  * @param pa_iRow      row of cell
  * @return             amount of neighbours 
  */
 private int countNeighbours(int pa_iColumn, int pa_iRow)
 {
  int lc_iCount = 0;
  
  int lc_iX1 = pa_iColumn - 1;
  int lc_iX2 = pa_iColumn    ;
  int lc_iX3 = pa_iColumn + 1;
  int lc_iY1 = pa_iRow    - 1;
  int lc_iY2 = pa_iRow       ;
  int lc_iY3 = pa_iRow    + 1;

  //
  // moore and von neumann have those 4 neighbours in common
  //
  lc_iCount += checkNeighbourCell(lc_iX2, lc_iY1);
  lc_iCount += checkNeighbourCell(lc_iX1, lc_iY2);
  lc_iCount += checkNeighbourCell(lc_iX3, lc_iY2);
  lc_iCount += checkNeighbourCell(lc_iX2, lc_iY3);
  
  //
  // the next 4 neighbours only count if we are in moore mode
  //
  if (m_tRuleset.getNeighbourHood() == GridLifeConstants.GOL_NEIGHBOURHOOD_MOORE)
  {
   lc_iCount += checkNeighbourCell(lc_iX1, lc_iY1);
   lc_iCount += checkNeighbourCell(lc_iX3, lc_iY1);
   lc_iCount += checkNeighbourCell(lc_iX1, lc_iY3);
   lc_iCount += checkNeighbourCell(lc_iX3, lc_iY3);
  }
  return lc_iCount;
 }
 
 
 
 /**
  * Checks the given neighbour cell
  * 
  * @param pa_iX   neighbour cell column
  * @param pa_iY   neighbour cell row
  * @return  1, if we have a valid living neighbour, 0 otherwise
  */
 private int checkNeighbourCell(int pa_iX, int pa_iY)
 {
  int     lc_iResult  = 0;
  boolean lc_bIsTorus = (m_tRuleset.getTopology() == GridLifeConstants.GOL_TOPOLOGY_TORUS);
  
  int lc_iX = pa_iX;
  int lc_iY = pa_iY;
  
  int lc_iMaxColumn = m_tRuleset.getColumns() -1;
  int lc_iMaxRow    = m_tRuleset.getRows()    -1;
  
  //
  // handle torus
  //
  if (true == lc_bIsTorus)
  {
   if (lc_iX > lc_iMaxColumn) lc_iX = 0; 
   if (lc_iX < 0            ) lc_iX = lc_iMaxColumn;  
   if (lc_iY > lc_iMaxRow   ) lc_iY = 0; 
   if (lc_iY < 0            ) lc_iY = lc_iMaxRow   ;
   
   lc_iResult = (0 == m_tCellArray[lc_iY][lc_iX] ? 0 : 1); 
  }
  //
  // handle sheet
  //
  else
  {
   if (lc_iX >= 0 && lc_iX < m_tRuleset.getColumns())
   {
    if (lc_iY >= 0 && lc_iY < m_tRuleset.getRows())
    {
     lc_iResult = (0 == m_tCellArray[lc_iY][lc_iX] ? 0 : 1); 
    }
   }
  }
  return lc_iResult;
 }

 
 /**
  * Returns the work array
  * 
  * @return null in case of error, work array otherwise
  */
 public final int[][] getWorkArray() {return m_tWorkArray;}

 
 
 /**
  * Returns the start column of the area
  * 
  * @return   start column
  */
 public final int getStartX() {return m_iStartX;}

 
 
 /**
  * Returns the start row of the area
  * 
  * @return   start row
  */ 
 public final int getStartY() {return m_iStartY;}

 
 
 /**
  * Returns the amount of columns of the area
  * 
  * @return   amount of  columns
  */
 public final int getColumns() {return m_iColumns;}


 
 /**
  * Returns the amount of rows of the area
  * 
  * @return   amount of  rows
  */
 public final int getRows() {return m_iRows;}


 
 /**
  * Returns the amount of cells in the work array
  * 
  * @return amount of cells
  */
 
 public final int getAmount() {return m_iAmount;}

 
 
 /**
  * Copies the computed results into the main cell array
  * 
  * @return amount of cells in the cell array area
  */
 public final int writeBackResults()
 {
  int lc_iRow    = 0;
  int lc_iColumn = 0;
  int lc_iAmount = 0;
  
  if (null != m_tCellArray && null != m_tWorkArray & null != m_tRuleset) 
  {
   for (lc_iRow = 0; lc_iRow < m_iRows; lc_iRow++) 
   {
    for (lc_iColumn = 0; lc_iColumn < m_iColumns; lc_iColumn++) 
    {
     m_tCellArray[m_iStartY + lc_iRow][m_iStartX + lc_iColumn] = m_tWorkArray[lc_iRow][lc_iColumn];
     if (0 != m_tWorkArray[lc_iRow][lc_iColumn])
     {
      lc_iAmount++; 
     }
    }
   }
  }
  return lc_iAmount;
 }
 
 
 /**
  * Returns the age histogram mananager
  * 
  * @return 
  */
 public final GridLifeAgeHistogramManager getHistogramManager() {return m_tAgeHistogramManager;}
 
 
} // eoc
