/**
 * Game of Life
 * 
 */
package gridlife.core;



/**
 * Imports
 * 
 */
import java.util.StringTokenizer;
import java.io.File             ;
import java.io.FileReader       ;
import java.io.FileWriter       ;
import java.io.BufferedReader   ;
import java.io.BufferedWriter   ;
import java.awt.Color           ;



/**
 * This class contains the properties 
 *
 * @author Ulf Wagemann
 */
public class GridLifeProperties
{
 private int m_iColumns          ;
 private int m_iRows             ;
 private int m_iNeighbourHoodType;
 private int m_iTopology         ;
 
 private int m_iAnimationDelay     ;
 private int m_iShowHelpLines      ;
 private int m_iShowGridLines      ;
 private int m_iShowCellBorders    ;
 private int m_iShowCellHiliteLines;
 private int m_iInvertMouseWheel   ;
 private int m_iScreenshotCellSize ;
 
 private int[] m_tCellStayAliveValues  ;
 private int[] m_tCellBecomeAliveValues;
 
 private Color  m_tColorBg                ;
 private Color  m_tColorGridLines         ;
 private Color  m_tColorHelpLines         ;
 private Color  m_tColorCellHilightLines  ;
 private Color  m_tColorCellBorders       ;
 private Color  m_tColorCellSelectionFrame;
 private Color  m_tColorCellSelectionArea ;
 private Color  m_tColorCellInsertionArea ;
 private Color  m_tColorCellInsertionFrame;
 
 
 
/**
 * Constructor
 *
 */
 public GridLifeProperties()
 {
  setColumns        (0);
  setRows           (0);
  setTopology       (0);
  setAnimationDelay (125);
  setShowHelpLines  (0);
  setShowGridLines  (0);
  setNeighbourHood  (0);
  setShowCellHiliteLines(0);
  setShowCellBorders(0);
  setScreenshotCellSize(3);
  setInvertMouseWheel(0);
  
  m_tCellStayAliveValues   = null;
  m_tCellBecomeAliveValues = null;

  setDefaultColors();
 }

 
 /**
  * Constructor. Very ugly, but very useful.
  * 
  * @param pa_iStayAlive                       ruleset for cells to stay alive
  * @param pa_iBecomeAlive                     ruleset for cells to become alive
  * @param pa_iNeighbourhoodType               neightbourhood type
  * @param pa_iColumns                         amount of columns 
  * @param pa_iRows                            amount of rows
  * @param pa_iTopology                        topology
  * @param pa_AnimDelay                        animation delay in ms
  * @param pa_iShowHelpLines                   flag defining which helplines are drawn
  * @param pa_iShowGridLines                   flag defining whether gridlines are drawn
  * @param pa_iShowCellHiliteLines             flag defining whether cell hilights are drawn
  * @param pa_iShowCellBorders                 flag defining whether cell borders are drawn 
  * @param pa_tColorBg                         background (=dead cell) color
  * @param pa_tColorGridLines                  gridline color
  * @param pa_tColorHelpLines                  helpline color
  * @param pa_tColorCellHilightLines           cell hilight color
  * @param pa_tColorCellBorders                cell border color
  * @param pa_tColorCellSelectionFrame         selection frame color
  * @param pa_tColorCellSelectionArea          selection area color
  * @param pa_tColorCellInsertionFrame         insertion frame color
  * @param pa_tColorCellInsertionArea          insertion area color
  * @param pa_iScreenshotCellSize              screenshot cell size in px
  * @param pa_iInvertMouseWheel                invert mouse wheel
  */
 public GridLifeProperties(int[] pa_iStayAlive              ,
                           int[] pa_iBecomeAlive            ,  
                           int   pa_iNeighbourhoodType      ,   
                           int   pa_iColumns                , 
                           int   pa_iRows                   , 
                           int   pa_iTopology               ,   
                           int   pa_AnimDelay               ,
                           int   pa_iShowHelpLines          ,
                           int   pa_iShowGridLines          ,
                           int   pa_iShowCellHiliteLines    ,
                           int   pa_iShowCellBorders        ,
                           Color pa_tColorBg                ,
                           Color pa_tColorGridLines         ,
                           Color pa_tColorHelpLines         ,
                           Color pa_tColorCellHilightLines  ,
                           Color pa_tColorCellBorders       ,
                           Color pa_tColorCellSelectionFrame,
                           Color pa_tColorCellSelectionArea ,
                           Color pa_tColorCellInsertionFrame,
                           Color pa_tColorCellInsertionArea ,
                           int   pa_iScreenshotCellSize     ,
                           int   pa_iInvertMouseWheel       )
 {
  setNeighbourHood (pa_iNeighbourhoodType);
  setColumns       (pa_iColumns          );
  setRows          (pa_iRows             );
  setTopology      (pa_iTopology         );
  setAnimationDelay(pa_AnimDelay         );
  setShowHelpLines (pa_iShowHelpLines    );
  setShowGridLines (pa_iShowGridLines    );
  setShowCellHiliteLines(pa_iShowCellHiliteLines);
  setShowCellBorders    (pa_iShowCellBorders    );
  setInvertMouseWheel   (pa_iInvertMouseWheel   );
  
  setColorBg                (pa_tColorBg                );
  setColorGridLines         (pa_tColorGridLines         );
  setColorHelpLines         (pa_tColorHelpLines         );
  setColorCellHilightLines  (pa_tColorCellHilightLines  );
  setColorCellBorders       (pa_tColorCellBorders       );
  setColorCellSelectionFrame(pa_tColorCellSelectionFrame);
  setColorCellSelectionArea (pa_tColorCellSelectionArea );
  setColorCellInsertionFrame(pa_tColorCellInsertionFrame);
  setColorCellInsertionArea (pa_tColorCellInsertionArea );
  
  setScreenshotCellSize(pa_iScreenshotCellSize);
  setRules(pa_iStayAlive, pa_iBecomeAlive);
 }

  
          
 /**
  * Constructor
  *
  * @param pa_tProperties    properties to use for initialization
  */
  public GridLifeProperties(GridLifeProperties pa_tProperties)
 {
  if (null != pa_tProperties)
  {
   copyRules(pa_tProperties);
  }
 }

  
  
 /**
  * Returns the ruleset for a cell to stay alive
  * 
  * @return int array
  */
  public final int[] getStayAliveRules()   {return m_tCellStayAliveValues;}
  
  
  
 /**
  * Returns the ruleset for a cell to become alive
   * 
  * @return int array
  */
  public final int[] getBecomeAliveRules() {return m_tCellBecomeAliveValues;}
  
  
  
 /**
   * Sets the ruleset for a cell to stay alive
   * 
   * @param pa_iStayAlive    int array
   * @param pa_iBecomeAlive  int array
   */
  public final void setRules(int[] pa_iStayAlive,int[] pa_iBecomeAlive)
  {
   setStayAliveRules  (pa_iStayAlive  );
   setBecomeAliveRules(pa_iBecomeAlive);
  }
          

  
  /**
   * Sets the rules for a cell to stay alive to the given array. 
   * The internal array is reallocated and filled with the given contents.
   * 
   * @param pa_iRules  int array
   */
  public final void setStayAliveRules(int[] pa_iRules)
  {
   int     lc_iLength = 0   ;
   int     lc_iI      = 0   ;
  
   if (null != pa_iRules)
   {
    if (0 < (lc_iLength = pa_iRules.length)) 
    {
     if (null != (m_tCellStayAliveValues = new int[lc_iLength]))
     {
      for (lc_iI = 0; lc_iI < lc_iLength; lc_iI++)
      {
       m_tCellStayAliveValues[lc_iI] = pa_iRules[lc_iI]; 
      }
     }
    }
   }
  }
  
  
  
  /**
   * Sets the rules for a cell to stay alive to the given array. 
   * The internal array is reallocated and filled with the given contents.
   * 
   * @param pa_iRules  int array
   */
  public final void setBecomeAliveRules(int[] pa_iRules)
  {
   int     lc_iLength = 0   ;
   int     lc_iI      = 0   ;
  
   if (null != pa_iRules)
   {
    if (0 < (lc_iLength = pa_iRules.length)) 
    {
     if (null != (m_tCellBecomeAliveValues = new int[lc_iLength]))
     {
      for (lc_iI = 0; lc_iI < lc_iLength; lc_iI++)
      {
       m_tCellBecomeAliveValues[lc_iI] = pa_iRules[lc_iI]; 
      }
     }
    }
   }
  }
  
  
  
 /**
  * Copies the values from the parameter object to the given object. Does not care about the L&F name!
  * 
  * @param pa_tProperties   Properties to copy values from
  */
 public final void copyRules(GridLifeProperties pa_tProperties)
 {
  if (null != pa_tProperties)
  {
   m_iInvertMouseWheel    = pa_tProperties.getInvertMouseWheel()   ;
   m_iScreenshotCellSize  = pa_tProperties.getScreenshotCellSize() ;
   m_iColumns             = pa_tProperties.getColumns()            ;
   m_iRows                = pa_tProperties.getRows()               ;
   m_iNeighbourHoodType   = pa_tProperties.getNeighbourHood()      ;   
   m_iTopology            = pa_tProperties.getTopology()           ;
   m_iAnimationDelay      = pa_tProperties.getAnimationDelay()     ;
   m_iShowHelpLines       = pa_tProperties.getShowHelpLines()      ;
   m_iShowGridLines       = pa_tProperties.getShowGridLines()      ;
   m_iShowCellHiliteLines = pa_tProperties.getShowCellHiliteLines();
   m_iShowCellBorders     = pa_tProperties.getShowCellBorders()    ;
   
   m_tColorBg                 = new Color(pa_tProperties.getColorBg().getRGB()                );
   m_tColorGridLines          = new Color(pa_tProperties.getColorGridLines().getRGB()         );
   m_tColorHelpLines          = new Color(pa_tProperties.getColorHelpLines().getRGB()         );
   m_tColorCellHilightLines   = new Color(pa_tProperties.getColorCellHilightLines().getRGB()  );
   m_tColorCellBorders        = new Color(pa_tProperties.getColorCellBorders().getRGB()       );
   m_tColorCellSelectionFrame = new Color(pa_tProperties.getColorCellSelectionFrame().getRGB());
   m_tColorCellSelectionArea  = new Color(pa_tProperties.getColorCellSelectionArea().getRGB() );
   m_tColorCellInsertionFrame = new Color(pa_tProperties.getColorCellInsertionFrame().getRGB());
   m_tColorCellInsertionArea  = new Color(pa_tProperties.getColorCellInsertionArea().getRGB() );

   setRules(pa_tProperties.getStayAliveRules(), pa_tProperties.getBecomeAliveRules());
  }
 }
 
 
 
 /**
  * Checks whether the two rulesets are equal or have the same content. Does not care about the L&F name!
  * 
  * @param pa_tProperties
  * @return false if they are not equals and have not the same content, otherwise true
  */
 public final boolean isEqual(GridLifeProperties pa_tProperties)
 {
  boolean lc_bEqual = false;
  
  if (null != pa_tProperties) 
  {
   lc_bEqual = true;
    
   lc_bEqual &= (pa_tProperties.getInvertMouseWheel()    == m_iInvertMouseWheel   );
   lc_bEqual &= (pa_tProperties.getScreenshotCellSize()  == m_iScreenshotCellSize );
   lc_bEqual &= (pa_tProperties.getColumns()             == m_iColumns            );
   lc_bEqual &= (pa_tProperties.getRows()                == m_iRows               );
   lc_bEqual &= (pa_tProperties.getNeighbourHood()       == m_iNeighbourHoodType  );
   lc_bEqual &= (pa_tProperties.getTopology()            == m_iTopology           );
   lc_bEqual &= (pa_tProperties.getAnimationDelay()      == m_iAnimationDelay     );
   lc_bEqual &= (pa_tProperties.getShowHelpLines()       == m_iShowHelpLines      );
   lc_bEqual &= (pa_tProperties.getShowGridLines()       == m_iShowGridLines      );
   lc_bEqual &= (pa_tProperties.getShowCellHiliteLines() == m_iShowCellHiliteLines);
   lc_bEqual &= (pa_tProperties.getShowCellBorders()     == m_iShowCellBorders    );
   
   lc_bEqual &= (pa_tProperties.getColorBg().getRGB()                 == m_tColorBg.getRGB()                );
   lc_bEqual &= (pa_tProperties.getColorGridLines().getRGB()          == m_tColorGridLines.getRGB()         );
   lc_bEqual &= (pa_tProperties.getColorHelpLines().getRGB()          == m_tColorHelpLines.getRGB()         );
   lc_bEqual &= (pa_tProperties.getColorCellHilightLines().getRGB()   == m_tColorCellHilightLines.getRGB()  );
   lc_bEqual &= (pa_tProperties.getColorCellBorders().getRGB()        == m_tColorCellBorders.getRGB()       );
   lc_bEqual &= (pa_tProperties.getColorCellSelectionFrame().getRGB() == m_tColorCellSelectionFrame.getRGB());
   lc_bEqual &= (pa_tProperties.getColorCellSelectionArea().getRGB()  == m_tColorCellSelectionArea.getRGB() );
   lc_bEqual &= (pa_tProperties.getColorCellInsertionArea().getRGB()  == m_tColorCellInsertionArea.getRGB() );
   lc_bEqual &= (pa_tProperties.getColorCellInsertionFrame().getRGB() == m_tColorCellInsertionFrame.getRGB());
   
   lc_bEqual &= compareRules(pa_tProperties.getStayAliveRules()  , getStayAliveRules()  );
   lc_bEqual &= compareRules(pa_tProperties.getBecomeAliveRules(), getBecomeAliveRules());
  }
  return lc_bEqual;
 }

 
 
 /**
  * Compares whether the 2 arrays are of equal size and content
  * 
  * @param pa_tRules1   int array
  * @param pa_tRules2   int array
  * @return  true, if sizes and contents are equals, false otherwise
  */
 private boolean compareRules(int[] pa_tRules1, int[] pa_tRules2)
 {
  boolean lc_bEqual  = true;
  int     lc_iLength = 0   ;
  int     lc_iI      = 0   ;
  
  if (null == pa_tRules1 && null == pa_tRules2)
  {
   return true; 
  }
  else
  {
   if (null != pa_tRules1 && null != pa_tRules2) 
   {
    if (0 < (lc_iLength = pa_tRules1.length))
    {
     if (lc_iLength == pa_tRules2.length) 
     {
      for (lc_iI = 0; lc_iI < lc_iLength; lc_iI++)
      {
       lc_bEqual &= (pa_tRules1[lc_iI] == pa_tRules2[lc_iI]); 
      }
      return lc_bEqual;
     }
    }
   }
  }
  return false;
 }
 

 
 /**
  * Returns the nouse wheel inversion flag
  * 
  * @return  nouse wheel inversion flag
  */
 public final int getInvertMouseWheel()
 {
  return m_iInvertMouseWheel;  
 }
 
 
 /**
  * Returns the screenshot pixel size
  * 
  * @return  screenshot pixel size
  */
 public final int getScreenshotCellSize()
 {
  return m_iScreenshotCellSize;
 }
 
 
 
 /**
  * Returns the nouse wheel inversion flag
  * 
  * @return  nouse wheel inversion flag
  */
 public final void setInvertMouseWheel(int pa_iValue)
 {
  if (0 <= pa_iValue && pa_iValue <= 1) 
  {
   m_iInvertMouseWheel = pa_iValue;  
  }
 }
 
 
 
 /**
  * Sets the screenshot pixel size
  * 
  * @param pa_iSize   screenshot pixel size
  */
 public final void setScreenshotCellSize(int pa_iSize)
 {
  m_iScreenshotCellSize = pa_iSize;
 }
 
 
 
 /**
  * Returns the background color of the cell panel
  * 
  * @return  background color
  */
 public final Color getColorBg() {return m_tColorBg;}
 
 

 /**
  * Sets the background color of the cell panel
  * 
  * @param pa_tColor   background color
  */
 public final void setColorBg(Color pa_tColor)
 {
  if (null != pa_tColor)
  {
   m_tColorBg = new Color(pa_tColor.getRGB()); 
  }
 }



 /**
  * Returns the cell selection frame color
  * 
  * @return  cell selection frame color
  */
 public final Color getColorCellSelectionFrame() {return m_tColorCellSelectionFrame;}
 
 

 /**
  * Sets the cell selection frame
  * 
  * @param pa_tColor   cell selection frame color
  */
 public final void setColorCellSelectionFrame(Color pa_tColor)
 {
  if (null != pa_tColor)
  {
   m_tColorCellSelectionFrame = new Color(pa_tColor.getRGB()); 
  }
 }
 
 
 
 /**
  * Returns the cell insertion frame color
  * 
  * @return  cell selection frame color
  */
 public final Color getColorCellInsertionFrame() {return m_tColorCellInsertionFrame;}
 
 

 /**
  * Sets the cell insertion frame
  * 
  * @param pa_tColor   cell selection frame color
  */
 public final void setColorCellInsertionFrame(Color pa_tColor)
 {
  if (null != pa_tColor)
  {
   m_tColorCellInsertionFrame = new Color(pa_tColor.getRGB()); 
  }
 }

 
 
 
 /**
  * Returns the cell selection area color
  * 
  * @return  cell selection area color
  */
 public final Color getColorCellSelectionArea() {return m_tColorCellSelectionArea;}
 
 

 /**
  * Sets the cell selection area
  * 
  * @param pa_tColor   cell selection area color
  */
 public final void setColorCellSelectionArea(Color pa_tColor)
 {
  if (null != pa_tColor)
  {
   m_tColorCellSelectionArea = new Color(pa_tColor.getRGB()); 
  }
 }
 
 
 
 /**
  * Returns the cell insertion area color
  * 
  * @return  cell insertion area color
  */
 public final Color getColorCellInsertionArea() {return m_tColorCellInsertionArea;}
 
 

 /**
  * Sets the cell insertion area
  * 
  * @param pa_tColor   cell insertion area color
  */
 public final void setColorCellInsertionArea(Color pa_tColor)
 {
  if (null != pa_tColor)
  {
   m_tColorCellInsertionArea = new Color(pa_tColor.getRGB()); 
  }
 }

 
 
 /**
  * Returns the cell border color
  * 
  * @return  cell border color
  */
 public final Color getColorCellBorders() {return m_tColorCellBorders;}
 
 

 /**
  * Sets the cell border color
  * 
  * @param pa_tColor  cell border color
  */
 public final void setColorCellBorders(Color pa_tColor)
 {
  if (null != pa_tColor)
  {
   m_tColorCellBorders = new Color(pa_tColor.getRGB()); 
  }
 }

 
 
 /**
  * Returns the gridline color of the cell panel
  * 
  * @return  gridline color
  */
 public final Color getColorGridLines() {return m_tColorGridLines;}

 
 
 /**
  * Sets the gridlines color
  * 
  * @param pa_tColor  gridline color
  */
 public final void setColorGridLines(Color pa_tColor)
 {
  if (null != pa_tColor)
  {
   m_tColorGridLines = new Color(pa_tColor.getRGB()); 
  }
 }

 
 
 /**
  * Returns the helplines color of the cell panel
  * 
  * @return  helplines color
  */
 public final Color getColorHelpLines() {return m_tColorHelpLines;}


 
 /**
  * Sets the helplines color
  * 
  * @param pa_tColor   helplines color
  */
 public final void setColorHelpLines(Color pa_tColor)
 {
  if (null != pa_tColor)
  {
   m_tColorHelpLines = new Color(pa_tColor.getRGB()); 
  }
 }

 
 
 /**
  * Returns the cell hilight lines color of the cell panel
  * 
  * @return  cell hilight color
  */
 public final Color getColorCellHilightLines() {return m_tColorCellHilightLines;}

 
 
 /**
  * Sets the cell hilight lines color
  * 
  * @param pa_tColor  cell hilight color
  */
 public final void setColorCellHilightLines(Color pa_tColor)
 {
  if (null != pa_tColor)
  {
   m_tColorCellHilightLines = new Color(pa_tColor.getRGB()); 
  }
 }

 
 /**
  * Returns cell hilite lines settings
  * 
  * @return   1 if cell hilite are to be shown, 0 otherwise
  */
 public final int getShowCellHiliteLines() {return m_iShowCellHiliteLines;};
 
  
 
 /**
  * Sets whether cell hilite lines should be shown or not
  * 
  * @param pa_iValue   1 to show cell hilite lines,0 to hide them
  */
 public final void setShowCellHiliteLines(int pa_iValue)
 {
  if (0 <= pa_iValue && pa_iValue <= 1) 
  {
   m_iShowCellHiliteLines = pa_iValue; 
  }
 }


 
 /**
  * Returns cell hilite lines settings
  * 
  * @return   1 if cell hilite are to be shown, 0 otherwise
  */
 public final int getShowCellBorders() {return m_iShowCellBorders;};
 
  
 
 /**
  * Sets whether cell hilite lines should be shown or not
  * 
  * @param pa_iValue   1 to show cell hilite lines,0 to hide them
  */
 public final void setShowCellBorders(int pa_iValue)
 {
  if (0 <= pa_iValue && pa_iValue <= 1) 
  {
   m_iShowCellBorders = pa_iValue; 
  }
 }

 
 
 /**
  * Returns gridline settings
  * 
  * @return   1 if gridlines are to be shown, 0 otherwise
  */
 public final int getShowGridLines() {return m_iShowGridLines;};
 
  
 
 /**
  * Setswhether gridlines should be shown or not
  * 
  * @param pa_iValue   1 to show gridlines,0 to hide them
  */
 public final void setShowGridLines(int pa_iValue)
 {
  if (0 <= pa_iValue && pa_iValue <= 1) 
  {
   m_iShowGridLines = pa_iValue; 
  }
 }
 
 
 
  /**
  * Returns the amount of columns
  * 
  * @return   Number of columns
  */
 public final int getColumns() {return m_iColumns;};
 
  
 
 /**
  * Sets the amount of rows to the given value
  * 
  * @param pa_iValue   new amount of rows
  */
 public final void setColumns(int pa_iValue)
 {
  if (pa_iValue >= GridLifeConstants.GOL_MIN_COLUMNS && pa_iValue <= GridLifeConstants.GOL_MAX_COLUMNS) 
  {
   m_iColumns = pa_iValue; 
  }
 }

 
 
 /**
  * Returns the amount of rows
  * 
  * @return   Number of rows
  */
 public final int getRows() {return m_iRows;};
 
 
 
 /**
  * Sets the amount of rows to the given value
  * 
  * @param pa_iValue   new amount of rows
  */
 public final void setRows(int pa_iValue)
 {
  if (pa_iValue >= GridLifeConstants.GOL_MIN_ROWS && pa_iValue <= GridLifeConstants.GOL_MAX_ROWS) 
  {
   m_iRows = pa_iValue; 
  }
 }


 /**
  * Retrieves the neighbourhood type
  * 
  * @return  GridLifeConstants.GOL_NEIGHBOURHOOD_VON_NEUMANN or GridLifeConstants.GOL_NEIGHBOURHOOD_MOORE
  */ 
 public final int getNeighbourHood() {return m_iNeighbourHoodType;};
 
 
 /**
  * Sets the neighbourhood type
  * 
  * @param pa_iValue   new type, must be either GridLifeConstants.GOL_NEIGHBOURHOOD_VON_NEUMANN or GridLifeConstants.GOL_NEIGHBOURHOOD_MOORE
  */ 
 public final void setNeighbourHood(int pa_iValue) 
 {
  if (GridLifeConstants.GOL_NEIGHBOURHOOD_VON_NEUMANN == pa_iValue || GridLifeConstants.GOL_NEIGHBOURHOOD_MOORE == pa_iValue)
  {
   m_iNeighbourHoodType = pa_iValue;
  }
 }


 /**
  * Returns the topology
  * 
  * @return    topology (0 = sheet, 1 = torus)
  */
 public final int getTopology() {return m_iTopology;}
 
 
 
 /**
  * Sets the topology to the given value
  * 
  * @param pa_iValue   topology (0 = sheet, 1 = torus)
  */
 public final void setTopology(int pa_iValue)
 {
  m_iTopology = (GridLifeConstants.GOL_TOPOLOGY_SHEET == pa_iValue || GridLifeConstants.GOL_TOPOLOGY_TORUS == pa_iValue ? pa_iValue : GridLifeConstants.GOL_TOPOLOGY_SHEET); 
 }

 
 
 /**
  * Returns the animation delay in ms
  * 
  * @return   Animation delay > 0
  */
 public final int getAnimationDelay() {return m_iAnimationDelay;}
 
 
 
 /**
  * Sets the animation delay to the given value
  * 
  * @param pa_iDelay    delay
  */
 public final void setAnimationDelay(int pa_iDelay)
 {
  m_iAnimationDelay = (0 < pa_iDelay && GridLifeConstants.GOL_STAT_DEFAULT_MAX_DELAY > pa_iDelay ? pa_iDelay : 1);  
 }
 
 

 /**
  * Returns the helplines parameter
  * 
  * @return    helplines (0 = none, 1 = 50%, 2 = all)
  */
 public final int getShowHelpLines() {return m_iShowHelpLines;}
 
 
 
 
 /**
  * Sets the helplines paramater
  * 
  * @param pa_iValue   helplines (0 = none, 1 = 50%, 2 = all)
  */
 public final void setShowHelpLines(int pa_iValue)
 {
  m_iShowHelpLines = (GridLifeConstants.GOL_HELPLINES_NONE <= pa_iValue && 
                      GridLifeConstants.GOL_HELPLINES_FULL >= pa_iValue ? pa_iValue : GridLifeConstants.GOL_HELPLINES_NONE);
 } 

 
 
 /**
  * Checks whether at least one selection is active in every ruleset button row
  * 
  * @return  true, if the above is true, false otherwise
  */
 public final boolean hasValidRules()
 {
  boolean lc_bResult = false;
  
  int     lc_iCountStayAlive   = 0;
  int     lc_iCountBecomeAlive = 0;
  int     lc_iLength           = 0;
  int     lc_iI                = 0;
  
  if (null != m_tCellStayAliveValues  && null != m_tCellBecomeAliveValues)
  {
   if (0 < (lc_iLength = m_tCellStayAliveValues.length)) 
   {
    if (lc_iLength == m_tCellBecomeAliveValues.length) 
    {
     for (lc_iI = 0; lc_iI < lc_iLength; lc_iI++)
     {
      if (0 != m_tCellStayAliveValues  [lc_iI]) lc_iCountStayAlive++  ;
      if (0 != m_tCellBecomeAliveValues[lc_iI]) lc_iCountBecomeAlive++;
     }
     lc_bResult = (0 < lc_iCountStayAlive && 0 < lc_iCountBecomeAlive);
    }
   }
  }
  return lc_bResult;
 }
 
 
 
 /**
  * Extracts properties from a file string
  * 
  * @param pa_sLine   line from file
  * @return true, if extraction was successfull, false otherwise
  */
 private boolean extractData(String pa_sLine)
 {
  boolean         lc_bResult    = false;
  int             lc_iPos       = -1   ;
  int             lc_iI         = 0    ;
  int             lc_iLength    = 0    ;
  StringTokenizer lc_tTokenizer = null ;
  String          lc_sLine      = null ;  
  
  String lc_sRed   = "";
  String lc_sGreen = "";
  String lc_sBlue  = "";
  
  if (-1 != (lc_iPos = pa_sLine.indexOf(GridLifeConstants.GOL_FILE_PROPERTIES_HEADER)))          
  {
   lc_sLine = pa_sLine.substring(GridLifeConstants.GOL_FILE_PROPERTIES_HEADER.length());
   
   if (null != (lc_tTokenizer = new StringTokenizer(lc_sLine, GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR)))
   {
    try
    {
     m_iTopology            = Integer.valueOf(lc_tTokenizer.nextToken()); 
     m_iNeighbourHoodType   = Integer.valueOf(lc_tTokenizer.nextToken()); 
     m_iColumns             = Integer.valueOf(lc_tTokenizer.nextToken()); 
     m_iRows                = Integer.valueOf(lc_tTokenizer.nextToken()); 
     m_iAnimationDelay      = Integer.valueOf(lc_tTokenizer.nextToken()); 
     
     m_iShowHelpLines       = Integer.valueOf(lc_tTokenizer.nextToken()); 
     m_iShowGridLines       = Integer.valueOf(lc_tTokenizer.nextToken()); 
     m_iShowCellHiliteLines = Integer.valueOf(lc_tTokenizer.nextToken()); 
     m_iShowCellBorders     = Integer.valueOf(lc_tTokenizer.nextToken()); 
     
     lc_sRed    = lc_tTokenizer.nextToken();
     lc_sGreen  = lc_tTokenizer.nextToken();
     lc_sBlue   = lc_tTokenizer.nextToken();
     m_tColorBg = new Color(Integer.valueOf(lc_sRed).intValue(), Integer.valueOf(lc_sGreen).intValue(), Integer.valueOf(lc_sBlue).intValue());

     lc_sRed    = lc_tTokenizer.nextToken();
     lc_sGreen  = lc_tTokenizer.nextToken();
     lc_sBlue   = lc_tTokenizer.nextToken();
     m_tColorGridLines = new Color(Integer.valueOf(lc_sRed).intValue(), Integer.valueOf(lc_sGreen).intValue(), Integer.valueOf(lc_sBlue).intValue());

     lc_sRed    = lc_tTokenizer.nextToken();
     lc_sGreen  = lc_tTokenizer.nextToken();
     lc_sBlue   = lc_tTokenizer.nextToken();
     m_tColorHelpLines = new Color(Integer.valueOf(lc_sRed).intValue(), Integer.valueOf(lc_sGreen).intValue(), Integer.valueOf(lc_sBlue).intValue());

     lc_sRed    = lc_tTokenizer.nextToken();
     lc_sGreen  = lc_tTokenizer.nextToken();
     lc_sBlue   = lc_tTokenizer.nextToken();
     m_tColorCellHilightLines = new Color(Integer.valueOf(lc_sRed).intValue(), Integer.valueOf(lc_sGreen).intValue(), Integer.valueOf(lc_sBlue).intValue());

     lc_sRed    = lc_tTokenizer.nextToken();
     lc_sGreen  = lc_tTokenizer.nextToken();
     lc_sBlue   = lc_tTokenizer.nextToken();
     m_tColorCellBorders = new Color(Integer.valueOf(lc_sRed).intValue(), Integer.valueOf(lc_sGreen).intValue(), Integer.valueOf(lc_sBlue).intValue());

     lc_sRed    = lc_tTokenizer.nextToken();
     lc_sGreen  = lc_tTokenizer.nextToken();
     lc_sBlue   = lc_tTokenizer.nextToken();
     m_tColorCellSelectionFrame = new Color(Integer.valueOf(lc_sRed).intValue(), Integer.valueOf(lc_sGreen).intValue(), Integer.valueOf(lc_sBlue).intValue());

     lc_sRed    = lc_tTokenizer.nextToken();
     lc_sGreen  = lc_tTokenizer.nextToken();
     lc_sBlue   = lc_tTokenizer.nextToken();
     m_tColorCellSelectionArea = new Color(Integer.valueOf(lc_sRed).intValue(), Integer.valueOf(lc_sGreen).intValue(), Integer.valueOf(lc_sBlue).intValue());

     lc_sRed    = lc_tTokenizer.nextToken();
     lc_sGreen  = lc_tTokenizer.nextToken();
     lc_sBlue   = lc_tTokenizer.nextToken();
     m_tColorCellInsertionFrame = new Color(Integer.valueOf(lc_sRed).intValue(), Integer.valueOf(lc_sGreen).intValue(), Integer.valueOf(lc_sBlue).intValue());
     
     lc_sRed    = lc_tTokenizer.nextToken();
     lc_sGreen  = lc_tTokenizer.nextToken();
     lc_sBlue   = lc_tTokenizer.nextToken();
     m_tColorCellInsertionArea = new Color(Integer.valueOf(lc_sRed).intValue(), Integer.valueOf(lc_sGreen).intValue(), Integer.valueOf(lc_sBlue).intValue());
     
     lc_iLength               = 1 + (m_iNeighbourHoodType == GridLifeConstants.GOL_NEIGHBOURHOOD_MOORE ? 8 : 4);
     m_tCellStayAliveValues   = new int[lc_iLength];
     m_tCellBecomeAliveValues = new int[lc_iLength];
    
     if (null != m_tCellStayAliveValues && null != m_tCellBecomeAliveValues)  
     { 
      for (lc_iI = 0; lc_iI < lc_iLength; lc_iI++)
      {
       m_tCellStayAliveValues[lc_iI] = Integer.valueOf(lc_tTokenizer.nextToken());
      }
      for (lc_iI = 0; lc_iI < lc_iLength; lc_iI++)
      {
       m_tCellBecomeAliveValues[lc_iI] = Integer.valueOf(lc_tTokenizer.nextToken());
      }
      
      //
      // screenshot pixelsize in an own try/catch block to remain compatible to old files, where it might not exist
      //
      try
      {
       m_iScreenshotCellSize = Integer.valueOf(lc_tTokenizer.nextToken()); 
       m_iInvertMouseWheel   = Integer.valueOf(lc_tTokenizer.nextToken()); 
      }
      catch (Exception lc_tException) 
      {
       m_iScreenshotCellSize = 1;
       m_iInvertMouseWheel   = 0;
      }
      
      lc_bResult = true;
     }
    }
    catch (Exception lc_tException) {setDefaultColors();}
   }
  }
  return lc_bResult;
 }
 
 
 
 /**
  * Loads the properties file to the current object
  * 
  * @return true, if loading was successfull, false otherwise
  */
 public final boolean load(String pa_sFileName)
 {
  File           lc_tFile               = null ;
  FileReader     lc_tFileReader         = null ;
  BufferedReader lc_tBuffererFileReader = null ;
  String         lc_sLine               = null ;
  boolean        lc_bResult             = false;
  
  if (null != (lc_tFile = new File(pa_sFileName)))
  {
   try
   {
    if (null != (lc_tFileReader = new FileReader(lc_tFile)))   
    {
     if (null != (lc_tBuffererFileReader = new BufferedReader(lc_tFileReader)))   
     {
      //
      // write header and properties string
      //
      if (null != (lc_sLine = lc_tBuffererFileReader.readLine()))
      {
       lc_bResult = extractData(lc_sLine);
      }
     }
    }
   }
   catch (Exception lc_tException) {}
  }
    
  //
  // Closing file stuff at this position to make sure that in any case we close them!
  //
  try {if (null != lc_tBuffererFileReader) lc_tBuffererFileReader.close(); } catch (Exception lc_tException) {}   
  try {if (null != lc_tFileReader        ) lc_tFileReader.close();         } catch (Exception lc_tException) {}  
  
  return lc_bResult;
 }
 
 
 
 /**
  * Save the current properties to a file
  * 
  */
 public final void save(String pa_sName)
 {
  File           lc_tFile               = null;
  FileWriter     lc_tFileWriter         = null;
  BufferedWriter lc_tBuffererFileWriter = null;
   
  if (null != (lc_tFile = new File(pa_sName)))
  {
   try
   {
    if (null != (lc_tFileWriter = new FileWriter(lc_tFile)))   
    {
     if (null != (lc_tBuffererFileWriter = new BufferedWriter(lc_tFileWriter)))   
     {
      //
      // write header and properties string
      //
      lc_tBuffererFileWriter.write(GridLifeConstants.GOL_FILE_PROPERTIES_HEADER + this.toString());
     }
    }
   }
   catch (Exception lc_tException) {}
  }
    
  //
  // Closing file stuff at this position to make sure that in any case we close them!
  //
  try {if (null != lc_tBuffererFileWriter) lc_tBuffererFileWriter.close(); } catch (Exception lc_tException) {}   
  try {if (null != lc_tFileWriter        ) lc_tFileWriter.close();         } catch (Exception lc_tException) {}      
 }
 
 
 
 /**
  * toString implementation which is actually useful for sabing the settings in a compact form
  * 
  * @return string, null in case of error
  */
 @Override
 public String toString()
 {
  StringBuffer lc_tStringBuffer = new StringBuffer();
  int lc_iLength = 0;
  int lc_iI      = 0;
  
  if (null != m_tCellStayAliveValues && null != m_tCellBecomeAliveValues && null != lc_tStringBuffer)
  {
   lc_tStringBuffer.append(getTopology()     ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getNeighbourHood()); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColumns()      ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getRows()         ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   
   lc_tStringBuffer.append(getAnimationDelay()     ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getShowHelpLines()      ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getShowGridLines()      ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getShowCellHiliteLines()); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getShowCellBorders()    ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);

   lc_tStringBuffer.append(getColorBg().getRed()  ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorBg().getGreen()); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorBg().getBlue() ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);

   lc_tStringBuffer.append(getColorGridLines().getRed()  ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorGridLines().getGreen()); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorGridLines().getBlue() ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);

   lc_tStringBuffer.append(getColorHelpLines().getRed()  ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorHelpLines().getGreen()); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorHelpLines().getBlue() ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);

   lc_tStringBuffer.append(getColorCellHilightLines().getRed()  ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorCellHilightLines().getGreen()); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorCellHilightLines().getBlue() ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);

   lc_tStringBuffer.append(getColorCellBorders().getRed()  ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorCellBorders().getGreen()); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorCellBorders().getBlue() ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);

   lc_tStringBuffer.append(getColorCellSelectionFrame().getRed()  ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorCellSelectionFrame().getGreen()); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorCellSelectionFrame().getBlue() ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);

   lc_tStringBuffer.append(getColorCellSelectionArea().getRed()  ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorCellSelectionArea().getGreen()); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorCellSelectionArea().getBlue() ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);

   lc_tStringBuffer.append(getColorCellInsertionFrame().getRed()  ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorCellInsertionFrame().getGreen()); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorCellInsertionFrame().getBlue() ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   
   lc_tStringBuffer.append(getColorCellInsertionArea().getRed()  ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorCellInsertionArea().getGreen()); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getColorCellInsertionArea().getBlue() ); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   
   lc_iLength = 1 + (getNeighbourHood() == GridLifeConstants.GOL_NEIGHBOURHOOD_MOORE ? 8 : 4);
   
   for (lc_iI = 0; lc_iI < lc_iLength; lc_iI++)
   {
    lc_tStringBuffer.append(m_tCellStayAliveValues[lc_iI]); 
    lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   }
   for (lc_iI = 0; lc_iI < lc_iLength; lc_iI++)
   {
    lc_tStringBuffer.append(m_tCellBecomeAliveValues[lc_iI]); 
    lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   }
   
   lc_tStringBuffer.append(getScreenshotCellSize()); lc_tStringBuffer.append(GridLifeConstants.GOL_FILE_PROPERTIES_VALUE_SEPARATOR);
   lc_tStringBuffer.append(getInvertMouseWheel());
   
   return lc_tStringBuffer.toString();
  }
  return null;
 }


 /**
  * Sets default colors
  * 
  */
 private void setDefaultColors()
 {
  m_tColorBg                  = new Color(GridLifeConstants.GOL_CELL_BACKGROUND_COLOR             );
  m_tColorGridLines           = new Color(GridLifeConstants.GOL_CELL_GRID_COLOR                   );
  m_tColorHelpLines           = new Color(GridLifeConstants.GOL_CELL_GRID_HELPLINE_COLOR          );
  m_tColorCellHilightLines    = new Color(GridLifeConstants.GOL_CELL_HILITE_COLOR                 );
  m_tColorCellBorders         = new Color(GridLifeConstants.GOL_CELL_BORDER_COLOR                 );
  m_tColorCellSelectionArea   = new Color(GridLifeConstants.GOL_CELL_MATRIX_SELECTION_FILL_COLOR  );
  m_tColorCellSelectionFrame  = new Color(GridLifeConstants.GOL_CELL_MATRIX_SELECTION_BORDER_COLOR);
  m_tColorCellInsertionArea   = new Color(GridLifeConstants.GOL_CELL_MATRIX_INSERTION_AREA_COLOR  ); 
  m_tColorCellInsertionFrame  = new Color(GridLifeConstants.GOL_CELL_MATRIX_INSERTION_FRAME_COLOR ); 
 }
 
} // eoc

