/**
 * Game of Life
 * 
 */
package gridlife.core;



/**
 * Imports
 * 
 */
import java.util.Iterator  ;
import java.util.concurrent.LinkedBlockingDeque;



/**
 * This class keeps track of the statistics for easy use when drawing the population graph.
 * To avoid race-conditions caused my multi-threading, it uses a LinkedBlockingDeque.
 * 
 * @author Ulf Wagemann
 */
public class GridLifeStatisticsManager
{
 private LinkedBlockingDeque<GridLifeStatisticsRecord> m_tStatisticsRecords;
 private Iterator<GridLifeStatisticsRecord>   m_tIteratorRead     ;
 
 
 
 /**
  * Constructor
  * 
  */
 public GridLifeStatisticsManager()
 {
  m_tStatisticsRecords = new LinkedBlockingDeque<GridLifeStatisticsRecord>(GridLifeConstants.GOL_STAT_MAX_RECORDS); 
  m_tIteratorRead = null;
 }
 
 
 
 /**
  * Set memory free
  * 
  */
 public final void dispose()
 {
  if (null != m_tStatisticsRecords) 
  {
   m_tStatisticsRecords.clear();
   m_tStatisticsRecords = null ;
  }
 }
 
 
 
 /**
  * Adds a new record to the statistics
  * 
  * @param pa_lTimestamp   timestamp
  * @param pa_iCells       amount of cells
  * @param pa_lGeneration  generation
  * @param pa_iTendency    tendency
  * @param pa_sTendency    tendency string
  */
 public final void add(long pa_lTimestamp, int pa_iCells, long pa_lGeneration, int pa_iTendency, String pa_sTendency)
 {
  GridLifeStatisticsRecord lc_tRecord = new GridLifeStatisticsRecord(pa_lTimestamp, pa_iCells, pa_lGeneration, pa_iTendency, pa_sTendency); 
  GridLifeStatisticsRecord lc_tRemovedRecord = null;
  
  if (null != lc_tRecord && null != m_tStatisticsRecords)
  {
   if (m_tStatisticsRecords.size()+1 > GridLifeConstants.GOL_STAT_MAX_RECORDS)
   {
    lc_tRemovedRecord = m_tStatisticsRecords.poll(); 
    lc_tRemovedRecord = null;
   }
   try
   {
    m_tStatisticsRecords.put(lc_tRecord); 
   }
   catch (InterruptedException lc_tException)
   {}
  }
 }
 
 
 
 /**
  * Returns the amount of stored records
  * 
  * @return -1 in case of error, amount >=0 otherwise
  */
 public final int getSize()
 {
  return (null != m_tStatisticsRecords ? m_tStatisticsRecords.size() : -1); 
 }
 
 
 
 /**
  * Prepares the reading of the list from the end 
  * 
  * @return true, if reading can be continued
  */
 public final boolean startReading()
 {
  m_tIteratorRead = (null != m_tStatisticsRecords ? m_tStatisticsRecords.descendingIterator() : null); 
  return (null != m_tIteratorRead);
 }
 
 
 
 /**
  * Checks whether another entry can be read
  * 
  * @return true, if a following record exists, false otherwise
  */
 public final boolean hasReadNext()
 {
  return (null != m_tIteratorRead ? m_tIteratorRead.hasNext() : false); 
 }
 
 
 
 /**
  * Reads the next record entry
  * 
  * @return  null in case of error or end of list, record otherwise
  */
 public final GridLifeStatisticsRecord readNext()
 {
  if (null != m_tIteratorRead)
  {
   if (true == m_tIteratorRead.hasNext())
   {
    return m_tIteratorRead.next();
   }
  }
  return null;
 }
 
 

 /**
  * Clears the manager, removes all records
  * 
  */
 public final void clear()
 {
  m_tIteratorRead = null;
  if (null != m_tStatisticsRecords)
  {
   m_tStatisticsRecords.clear(); 
  }
 }
 
 
 
} // eoc
