/*
 * Game of Life
 * 
 */
package gridlife.ui;

import gridlife.core.GridLifeConstants;
import java.awt.Rectangle        ;
import java.io.File              ;
import java.io.FileInputStream   ;
import java.io.FileOutputStream  ;
import java.io.IOException       ;
import java.util.Properties      ;
import java.util.HashMap         ;
import java.util.Set             ;
import java.util.Iterator        ;
import javax.swing.JInternalFrame;



/**
 * This tiny class is responsible for bringing the internal frames into the same mode and position as
 * they were last time on program exit. Therefore, it keeps track of a properties file which it loads
 * and startup in order to open and position internal frames. At program exit, it stores the current
 * window modes and positions in this property file.
 * 
 * @author Ulf Wagemann
 */
public class GridLifeWindowSettings 
{
 private HashMap<String, JInternalFrame> m_tWindowMap ;
 private Properties                      m_tProperties;  
 private boolean                         m_bValid     ; 
 
 
 /**
  * Constructor. Loads properties file
  * 
  */
 public GridLifeWindowSettings()
 {
  m_tWindowMap  = new HashMap<String, JInternalFrame>(); 
  m_tProperties = new Properties();
  m_bValid      = false;
  
  loadProperties();
 }
 
 
 
 /**
  * Returns whether the settings are valid (= loaded)
  * 
  * @return  true if settings are valid, false otherwise
  */
 public final boolean isValid() {return m_bValid;}
 
 
  /**
  * We do not want to eat up memory, since we store own datastructures we need to remove them from the
  * hashmap and mark then as gone for the GC.
  * 
  * dispose() should be called when this shapeset is not needed anymore.
  * 
  */
 public final void dispose()
 {
  if (null != m_tWindowMap) {m_tWindowMap.clear();}
 }
 
 
 
 /**
  * Adds a frame under the given name to the internal map
  * 
  * @param pa_tFrame    Frame to add
  * @param pa_sName     Name to use
  */
 public final void addFrame(JInternalFrame pa_tFrame, String pa_sName)
 {
  if (null != m_tWindowMap)
  {
   if (null != pa_tFrame && null != pa_sName) 
   {
    if (false == pa_sName.isEmpty()) 
    {
     m_tWindowMap.put(pa_sName, pa_tFrame); 
    }
   }
  }
 }
 
 
 
 /**
  * Loads the properties file as defined in GridLifeConstants
  * 
  */
 private void loadProperties()
 {
  File             lc_tPropertiesFile = new File(GridLifeConstants.GOL_FILE_WINDOW_SETTINGS);
  FileInputStream  lc_tInputStream    = null;       
        
  if (null != lc_tPropertiesFile && null != m_tProperties)
  {
   if (true == lc_tPropertiesFile.exists() && true == lc_tPropertiesFile.isFile()) 
   {
    try
    {
     if (null != (lc_tInputStream = new FileInputStream(lc_tPropertiesFile)))
     {
      m_tProperties.load(lc_tInputStream);    
      lc_tInputStream.close();
      m_bValid = (false == m_tProperties.isEmpty());
     }
    }
    catch (IOException lc_tExcpetion)
    {}
   }
  }
 }
 
 
 
 /**
  * This routine move the frames according to the properties file
  * 
  */
 public final void adjustFrames()
 {
  Iterator       lc_tIterator = null;
  Set<String>    lc_tKeySet   = null;
  Object         lc_tObject   = null;
  String         lc_sKey      = null;
  String         lc_sValue    = null;
  JInternalFrame lc_tFrame    = null;
  Rectangle      lc_tBounds   = null;
  
  int lc_iX    = 0;
  int lc_iY    = 0;
  int lc_iW    = 0;
  int lc_iH    = 0;
  int lc_iOpen = 0;
  
  if (null != m_tProperties && null != m_tWindowMap)
  {
   if (null != (lc_tKeySet = m_tWindowMap.keySet()))
   {
    if (null != (lc_tIterator = lc_tKeySet.iterator()))
    {
     //
     // Loop over any window in the map
     //
     while (lc_tIterator.hasNext()) 
     {
      if (null != (lc_tObject = lc_tIterator.next()))
      {
       if (lc_tObject instanceof String)
       {
        if (null != (lc_sKey = (String) lc_tObject))
        {
         if (null != (lc_tFrame = m_tWindowMap.get(lc_sKey)))
         {
          //
          // get window data
          //
          lc_sValue = (String) m_tProperties.get(lc_sKey + GridLifeConstants.GOL_WINDOW_X);
          lc_iX     = getIntValue(lc_sValue);

          lc_sValue = (String) m_tProperties.get(lc_sKey + GridLifeConstants.GOL_WINDOW_Y);
          lc_iY     = getIntValue(lc_sValue);

          lc_sValue = (String) m_tProperties.get(lc_sKey + GridLifeConstants.GOL_WINDOW_W);
          lc_iW     = getIntValue(lc_sValue);

          lc_sValue = (String) m_tProperties.get(lc_sKey + GridLifeConstants.GOL_WINDOW_H);
          lc_iH     = getIntValue(lc_sValue);

          lc_sValue = (String) m_tProperties.get(lc_sKey + GridLifeConstants.GOL_WINDOW_OPEN);
          lc_iOpen     = getIntValue(lc_sValue);

          //
          // move, size and show window if appropiate
          //
          if (lc_iW > 0 && lc_iH > 0)
          {
           if (null != (lc_tBounds = new Rectangle()))
           {
            lc_tBounds.x      = lc_iX;
            lc_tBounds.y      = lc_iY;
            lc_tBounds.width  = lc_iW;
            lc_tBounds.height = lc_iH;
           
            lc_tFrame.setBounds(lc_tBounds);
           }        
           lc_tFrame.setVisible(1 == lc_iOpen);
          }
         }
        }
       }
      }
     } // while
    }
   }
  }   
 }
 
 

 /**
  * Retrieves an int value from a string
  * 
  * @param pa_sString   String
  * @return   int value
  * 
  */ 
 private int getIntValue(String pa_sString)
 {
  if (null != pa_sString)  
  {
   if (false == pa_sString.isEmpty()) 
   {
    return (new Integer(pa_sString)).intValue(); 
   }
  }
  return 0;
 }
 
 
 
 /**
  * This routine saves the settings for each of the stored JInternalFrames in the properties file
  * 
  * @param pa_sAnnotation Annotation
  */
 public final void saveProperties(String pa_sAnnotation)
 {
  File              lc_tPropertiesFile = new File(GridLifeConstants.GOL_FILE_WINDOW_SETTINGS);
  FileOutputStream  lc_tOutputStream   = null;       
        
  updateSettings();
  
  if (null != lc_tPropertiesFile && null != m_tProperties)
  {
   try
   {
    if (false == lc_tPropertiesFile.exists()) 
    {
     lc_tPropertiesFile.createNewFile();
    }
    
    if (true == lc_tPropertiesFile.exists() && true == lc_tPropertiesFile.isFile()) 
    {
     if (null != (lc_tOutputStream = new FileOutputStream(lc_tPropertiesFile)))
     {
      m_tProperties.store(lc_tOutputStream, pa_sAnnotation);  
      lc_tOutputStream.close();
     }
    }
   }
   catch (IOException lc_tException)
   {}
  }   
 }
 
 
 
 /**
  * Updates all settings according to the current JInternalFrames' states
  * 
  */
 private void updateSettings()
 {
  Iterator       lc_tIterator = null;
  Set<String>    lc_tKeySet   = null;
  Object         lc_tObject   = null;
  String         lc_sKey      = null;
  JInternalFrame lc_tFrame    = null;
  Rectangle      lc_tBounds   = null;
  
  if (null != m_tProperties && null != m_tWindowMap)
  {
   if (null != (lc_tKeySet = m_tWindowMap.keySet()))
   {
    if (null != (lc_tIterator = lc_tKeySet.iterator()))
    {
     //
     // Loop over any window in the map
     //
     while (lc_tIterator.hasNext()) 
     {
      if (null != (lc_tObject = lc_tIterator.next()))
      {
       if (lc_tObject instanceof String)
       {
        if (null != (lc_sKey = (String) lc_tObject))
        {
         if (null != (lc_tFrame = m_tWindowMap.get(lc_sKey)))
         {
          //
          // get window data. ATTENTION: writing anything other than strings to a props file leads to exceptions, that's why we
          // transform anything to strings using String.valueOf()
          //
          if (null != (lc_tBounds = lc_tFrame.getBounds())) 
          {
           m_tProperties.put(lc_sKey + GridLifeConstants.GOL_WINDOW_X, String.valueOf(lc_tBounds.x     ));
           m_tProperties.put(lc_sKey + GridLifeConstants.GOL_WINDOW_Y, String.valueOf(lc_tBounds.y     ));
           m_tProperties.put(lc_sKey + GridLifeConstants.GOL_WINDOW_W, String.valueOf(lc_tBounds.width ));
           m_tProperties.put(lc_sKey + GridLifeConstants.GOL_WINDOW_H, String.valueOf(lc_tBounds.height));
          }
          m_tProperties.put(lc_sKey + GridLifeConstants.GOL_WINDOW_OPEN, String.valueOf((true == lc_tFrame.isVisible() ? 1 : 0)));
         }
        }
       }
      }
     } // while
    }
   }
  }
 }
 

 
 /**
  * Returns a property value as string
  * 
  * @param pa_sKey   Property key
  * @return          null in case of error or undefined, string otherwise
  */
 public final String getPropertyValue(String pa_sKey)
 {
  Object lc_tObject = null;
  String lc_sResult = null;
  
  if (null != m_tProperties & null != pa_sKey)  
  {
   if (null != (lc_tObject = m_tProperties.get(pa_sKey)))
   {
    if (lc_tObject instanceof String) 
    {
     lc_sResult = lc_tObject.toString(); 
    }
   }
  }
  return lc_sResult;   
 }
 
 
 
 /**
  * Returns the value of the LoadPath property
  * 
  * @return   null in case of error or undefined, otherwise path
  */
 public final String getLoadPathPaletteFile()
 {
  return getPropertyValue(GridLifeConstants.GOL_SETTING_LOAD_PATH_PALETTE_FILE); 
 }

 
 
 /**
  * Sets the LoadPath property
  * 
  * @return   null in case of error or undefined, otherwise path
  */
 public final void setLoadPathPaletteFile(String pa_sPath)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_LOAD_PATH_PALETTE_FILE, pa_sPath); 
 }

 
  
 /**
  * Returns the value of the shape palette name property
  * 
  * @return   null in case of error or undefined, otherwise shape palette name
  */
 public final String getShapePaletteName()
 {
  return getPropertyValue(GridLifeConstants.GOL_SETTING_SHAPE_PALETTE); 
 }
 
 
 
 /**
  * Sets the shape palette name property
  * 
  * @param  pa_sValue   new value
  */
 public final void setShapePaletteName(String pa_sValue)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_SHAPE_PALETTE, pa_sValue); 
 }
 
 
 
 /**
  * Returns the value of the shape palette entry name property
  * 
  * @return   null in case of error or undefined, otherwise shape palette entry name
  */
 public final String getShapePaletteEntryName()
 {
  return getPropertyValue(GridLifeConstants.GOL_SETTING_SHAPE_PALETTE_ENTRY); 
 }

 
 
 /**
  * Sets the shape palette entry name property
  * 
  * @param  pa_sValue   new value
  */
 public final void setShapePaletteEntryName(String pa_sValue)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_SHAPE_PALETTE_ENTRY, pa_sValue); 
 }
 
 
 
 /**
  * Returns the value of the color palette name property
  * 
  * @return   null in case of error or undefined, otherwise color palette name
  */
 public final String getColorPaletteName()
 {
  return getPropertyValue(GridLifeConstants.GOL_SETTING_COLORSET_PALETTE); 
 }

 
 
 /**
  * Sets the color palette name property
  * 
  * @param  pa_sValue   new value
  */
 public final void setColorPaletteName(String pa_sValue)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_COLORSET_PALETTE, pa_sValue); 
 }
 
 
 
  
 /**
  * Returns the value of the color palette entry name property
  * 
  * @return   null in case of error or undefined, otherwise color palette entry name
  */
 public final String getColorPaletteEntryName()
 {
  return getPropertyValue(GridLifeConstants.GOL_SETTING_COLORSET_PALETTE_ENTRY); 
 }

 
 
 /**
  * Sets the color palette entry name property
  * 
  * @param  pa_sValue   new value
  */
 public final void setColorPaletteEntryName(String pa_sValue)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_COLORSET_PALETTE_ENTRY, pa_sValue); 
 }
 
 
 
 /**
  * Returns the value of the LoadPath property for palette files
  * 
  * @return   null in case of error or undefined, otherwise path
  */
 public final String getSavePathPaletteFile()
 {
  return getPropertyValue(GridLifeConstants.GOL_SETTING_SAVE_PATH_PALETTE_FILE); 
 }



 /**
  * Sets the LoadPath property for palette files
  * 
  * @return   null in case of error or undefined, otherwise path
  */
 public final void setSavePathPaletteFile(String pa_sPath)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_SAVE_PATH_PALETTE_FILE, pa_sPath); 
 }


 
 /**
  * Returns the value of the LoadPath property
  * 
  * @return   null in case of error or undefined, otherwise path
  */
 public final String getLoadPathCellFile()
 {
  return getPropertyValue(GridLifeConstants.GOL_SETTING_LOAD_PATH_CELL_FILE); 
 }
 
 
 
 /**
  * Sets the LoadPath property
  * 
  * @return   null in case of error or undefined, otherwise path
  */
 public final void setLoadPathCellFile(String pa_sPath)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_LOAD_PATH_CELL_FILE, pa_sPath); 
 }


 
 /**
  * Returns the value of the LoadPath property
  * 
  * @return   null in case of error or undefined, otherwise path
  */
 public final String getSavePathCellFile()
 {
  return getPropertyValue(GridLifeConstants.GOL_SETTING_SAVE_PATH_CELL_FILE); 
 }


 /**
  * Sets the LoadPath property
  * 
  * @return   null in case of error or undefined, otherwise path
  */
 public final void setSavePathCellFile(String pa_sPath)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_SAVE_PATH_CELL_FILE, pa_sPath); 
 }
 
 
 
 /**
  * Returns the value of the LoadPath property
  * 
  * @return   null in case of error or undefined, otherwise path
  */
 public final String getLoadPathPropertiesFile()
 {
  return getPropertyValue(GridLifeConstants.GOL_SETTING_LOAD_PATH_PROPERTIES); 
 }

 
 
 /**
  * Sets the LoadPath property
  * 
  * @return   null in case of error or undefined, otherwise path
  */
 public final void setLoadPathPropertiesFile(String pa_sPath)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_LOAD_PATH_PROPERTIES, pa_sPath); 
 } 
 
 
 
 /**
  * Returns the value of the LoadPath property
  * 
  * @return   null in case of error or undefined, otherwise path
  */
 public final String getSavePathPropertiesFile()
 {
  return getPropertyValue(GridLifeConstants.GOL_SETTING_SAVE_PATH_PROPERTIES); 
 }

 
 
 /**
  * Sets the LoadPath property
  * 
  * @return   null in case of error or undefined, otherwise path
  */
 public final void setSavePathPropertiesFile(String pa_sPath)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_SAVE_PATH_PROPERTIES, pa_sPath); 
 } 
 

 
 /**
  * Returns the LF name
  * 
  * @return   null in case of error or undefined, otherwise LF name
  */
 public final String getLookAndFeelName()
 {
  return getPropertyValue(GridLifeConstants.GOL_SETTING_LOOK_AND_FEEL); 
 }
 
 
 
 /**
  * Sets the LoadPath property
  * 
  * @return   null in case of error or undefined, otherwise path
  */
 public final void setLookAndFeelName(String pa_sName)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_LOOK_AND_FEEL, pa_sName); 
 }
 
 
 
 /**
  * Returns the zoom value
  * 
  * @return zoom value, may be 0
  */
 public final int getZoom() 
 {
  String lc_sZoom = getPropertyValue(GridLifeConstants.GOL_SETTING_ZOOM);

  return (null != lc_sZoom ? Integer.valueOf(lc_sZoom) : 0);
 }
 
 
 
 /**
  * Sets the zoom value
  * 
  * @param pa_iValue   new value
  */
 public final void setZoom(int pa_iValue)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_ZOOM, Integer.toString(pa_iValue));   
 }
 

 
 
 /**
  * Returns the fill rate value
  * 
  * @return zoom value, may be 0
  */
 public final int getFill() 
 {
  String lc_sFill = getPropertyValue(GridLifeConstants.GOL_SETTING_FILL);

  return (null != lc_sFill ? Integer.valueOf(lc_sFill) : 0);
 }
 
 
 
 /**
  * Sets the fill rate value
  * 
  * @param pa_iValue   new value
  */
 public final void setFill(int pa_iValue)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_FILL, Integer.toString(pa_iValue));   
 }
 
 
 
 /**
  * Returns the paint mode
  * 
  * @return paint mode value, may be 0
  */
 public final int getPaintMode() 
 {
  String lc_sZoom = getPropertyValue(GridLifeConstants.GOL_SETTING_PAINT_MODE);

  return (null != lc_sZoom ? Integer.valueOf(lc_sZoom) : 0);
 }
 
 
 
 /**
  * Sets the paint mode
  * 
  * @param pa_iValue   new value
  */
 public final void setPaintMode(int pa_iValue)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_PAINT_MODE, Integer.toString(pa_iValue));   
 } 



 /**
  * Returns the paint type
  * 
  * @return paint type value, may be 0
  */
 public final int getPaintType() 
 {
  String lc_sZoom = getPropertyValue(GridLifeConstants.GOL_SETTING_PAINT_TYPE);

  return (null != lc_sZoom ? Integer.valueOf(lc_sZoom) : 0);
 }
 
 
 
 /**
  * Sets the paint type
  * 
  * @param pa_iValue   new value
  */
 public final void setPaintType(int pa_iValue)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_PAINT_TYPE, Integer.toString(pa_iValue));   
 } 

 
 
 public final String getQuickSaveSlot()
 {
  return getPropertyValue(GridLifeConstants.GOL_SETTING_QUICK_SAVE_SLOT);  
 }
 
 
 
 public final void setQuickSaveSlot(String pa_sValue)
 {
  if (null != m_tProperties) m_tProperties.put(GridLifeConstants.GOL_SETTING_QUICK_SAVE_SLOT, (null != pa_sValue ? pa_sValue : ""));    
 }
} // eoc
